/**
 * @file
 * Test function to test a SPARQL query, as well as event listener to react to
 * the button click.
 *
 * In addition to this, the JS also prefills the required column in the presentation
 * tab. Not to mention the animated GIF it shows while loading!
 */

(function($, win) {
  'use strict';
  Drupal.behaviors.jsonpSparql = {
    attach: function (context, settings) {
      $(context).find('.jsonp-sparql__test-query').click(function() {
        // The delta of this form is the last character of the button ID.
        // One might wonder why one looks for ids of these, and the reason is
        // that all blocks used to be administered on one page.
        var delta = $(this).attr('id').slice(-1);
        var query = $('#edit-jsonp-sparql-sparql-' + delta).val();
        // Really quickly check if a random string will be inserted for the
        // template.
        var randomString = 'Punk aint no religious cult ' + Math.random() + 'Cesspools in eden' + Math.random();
        var replacedQuery = _.template(query, win.jsonpSparql.templateSettings)({
          value: randomString
        });
        // We do the replacing because people can use all kinds of variations
        // with templates here, for all we know.
        if (replacedQuery.indexOf(randomString) > 0) {
          // Well, that means we need to do the replacing.
          var placeholder = prompt(Drupal.t('Enter a value for the placeholder'), '');
          query = _.template(query, win.jsonpSparql.templateSettings)({
            value: placeholder
          });
        }
        var server = $('#edit-jsonp-sparql-server-' + delta).val();
        // The following might look hardcoded, and it kind of is. This is the
        // ajax path we registered with the .module file.
        var url = Drupal.settings.basePath + 'jsonp_sparql/ajax/test_sparql';
        // Get an animated GIF going to show we are loading stuff.
        $(this).after('<img id="removemeafter" src="' + Drupal.settings.basePath + Drupal.settings.jsonpSparql.modpath + '/img/loading.gif">')
        $.ajax({
          url: url,
          cache: false,
          type: 'POST',
          dataType: 'json',
          data: {
            query: query,
            url: server
          },
          success: function(data) {
            $('#removemeafter').remove();
            if (data.code == '200') {
              var message = '';
              if (!data.data || !data.data.results || !data.data.results.bindings) {
                // The response was probably not JSON.
                message = Drupal.t('We got a response, but no results. Maybe you used a valid URL, but not a valid SPARQL endpoint? Have you remembered to include the questionmark that goes before the query parameter?');
              }
              else {
                if (data.data.results.bindings.length > 0) {
                   message = Drupal.formatPlural(data.data.results.bindings.length, 'We got a response! The result gave us 1 row to use. Now go on with the configuration and make the site awesome.', 'We got a response! The result gave us @count rows to use. Now go on with the configuration and make the site awesome.');
                }
                else {
                   message = Drupal.t('We got a response, but with the placeholder you provided we got no rows to use. You might want to check your query (or maybe you had a typo when entering the placeholder. The good news is that the endpoint responded!')
                }
                var rows = '';
                var $templateEl = $('#edit-jsonp-sparql-data-template-' + delta);
                var value =  $templateEl.val();
                if (value.length < 1) {
                  console.log(data.data.head.vars)
                  $templateEl.val(rows);
                  message = message + '\n' + Drupal.t('The presentation tab has been updated with the names of the rows. You will most likely want to head over there to edit the presentation of each row.');
                }
              }
              alert(message)
            }
            else {
              alert(Drupal.t('Oh no. An error. The server returned an error code @code with the following message: @error', {'@code': data['code'], '@error': data['error']}));
            }
          }
        })
        return false;
      })
    }
  };
})(jQuery, window);
